@@ -17,7 +17,8 @@ log = logging.getLogger(__name__)
 log.setLevel(logging.DEBUG)
 
 error_messages = {
-    22: '[ERROR 22] The object was from a previous session. Please read the "Long Term Usage" section in README on github.'
+    22: '[ERROR 22] The object was from a previous session. Please read the "Long Term Usage" section in README on github.',
+    10: '[ERROR 10] Session has expired and pronotepy was not able to reinitialise the connection.'
 }
 
 
@@ -29,28 +30,36 @@ class Client(object):
     ----------
     pronote_url : str
         URL of the server
-    ent : bool
-        If the connection is from an ENT
+    username : str
+    password : str
     cookies : cookies
         Cookies for ENT connections
 
     Attributes
     ----------
-    start_day : str
+    start_day : datetime.datetime
         The first day of the school year
-    week : str
+    week : int
         The current week of the school year
+    logged_in : bool
+        If the user is successfully logged in
+
     """
 
-    def __init__(self, pronote_url, ent: bool = False, cookies=None, username: str = None, password: str = None):
+    def __init__(self, pronote_url, username: str = '', password: str = '', cookies=None):
         log.info('INIT')
         # start communication session
-        self.ent = ent
+        if not cookies and password == '' and username == '':
+            raise PronoteAPIError('Please provide login credentials. Cookies are None, and username and password are empty.')
         self.username = username
         self.password = password
         self.pronote_url = pronote_url
         self.communication = _Communication(pronote_url, cookies, self)
         self.attributes, self.func_options = self.communication.initialise()
+        if 'e' in self.attributes and 'f' in self.attributes:
+            self.ent = True
+        else:
+            self.ent = False
 
         # set up encryption
         self.encryption = _Encryption()
@@ -74,37 +83,27 @@ class Client(object):
         self.one_hour_duration = hour_end - hour_start
 
         self.periods_ = self.periods
-        if username and password:
-            self.logged_in = self.login(username, password)
-        elif ent and cookies:
-            self.logged_in = self.login()
-        else:
-            self.logged_in = False
+        self.logged_in = self._login()
+        self._expired = False
 
-    def login(self, username='', password=''):
+    def _login(self):
         """
         Logs in the user.
 
-        Parameters
-        ----------
-        username : str
-            Username
-        password : str
-            Password
-
         Returns
         -------
         bool
             True if logged in, False if not
         """
         if self.ent is True:
-            username = self.attributes['e']
-            password = self.attributes['f']
+            print('ent is true')
+            self.username = self.attributes['e']
+            self.password = self.attributes['f']
         # identification phase
         ident_json = {
             "genreConnexion": 0,
             "genreEspace": int(self.attributes['a']),
-            "identifiant": username,
+            "identifiant": self.username,
             "pourENT": self.ent,
             "enConnexionAuto": False,
             "demandeConnexionAuto": False,
@@ -125,17 +124,17 @@ class Client(object):
 
         # key gen
         if self.ent is True:
-            motdepasse = SHA256.new(str(password).encode()).hexdigest().upper()
+            motdepasse = SHA256.new(str(self.password).encode()).hexdigest().upper()
             e.aes_key = MD5.new(motdepasse.encode()).digest()
         else:
-            u = username
-            p = password
+            u = self.username
+            p = self.password
             if idr['donneesSec']['donnees']['modeCompLog']:
                 u = u.lower()
             if idr['donneesSec']['donnees']['modeCompMdp']:
                 p = p.lower()
             alea = idr['donneesSec']['donnees']['alea']
-            motdepasse = SHA256.new(str(alea + p).encode()).hexdigest().upper()
+            motdepasse = SHA256.new((alea + p).encode()).hexdigest().upper()
             e.aes_key = MD5.new((u + motdepasse).encode()).digest()
 
         # challenge
@@ -149,7 +148,7 @@ class Client(object):
         if 'cle' in self.auth_response.json()['donneesSec']['donnees']:
             self.communication.after_auth(self.auth_response, e.aes_key)
             self.encryption.aes_key = e.aes_key
-            log.info(f'successfully logged in as {username}')
+            log.info(f'successfully logged in as {self.username}')
             return True
         else:
             log.info('login failed')
@@ -296,22 +295,32 @@ class Client(object):
         return [dataClasses.Message(self, m) for m in messages.json()['donneesSec']['donnees']['listeMessagerie']['V']
                 if not m.get('estUneDiscussion')]
 
-    def _refresh(self):
+    def refresh(self):
         """
         Now this is the true jank part of this program. It refreshes the connection if something went wrong.
         This is the classical procedure if something is broken.
-        :return:
         """
         logging.debug('Reinitialisation')
+        self.communication.session.close()
         self.communication = _Communication(self.pronote_url, None, self)
         self.attributes, self.func_options = self.communication.initialise()
 
         # set up encryption
         self.encryption = _Encryption()
         self.encryption.aes_iv = self.communication.encryption.aes_iv
-        self.login(self.username, self.password)
+        self._login()
         self.periods_ = None
         self.periods_ = self.periods
+        self.week = self._get_week(datetime.date.today())
+        self._expired = True
+
+    def session_check(self) -> bool:
+        """Checks if the session has expired and refreshes it if it had (returns bool signifying if it was expired)"""
+        self.communication.post('Presence', {'_Signature_': {'onglet': 7}})
+        if self._expired:
+            self._expired = False
+            return True
+        return False
 
 
 class _Communication(object):
@@ -381,11 +390,16 @@ class _Communication(object):
         if not response.ok:
             raise requests.HTTPError(f'Status code: {response.status_code}')
         if 'Erreur' in response.json():
+            r_json = response.json()
+            if r_json["Erreur"]["G"] == 22:
+                raise ExpiredObject(error_messages.get(22))
             if recursive:
-                raise PronoteAPIError(error_messages.get(response.json()["Erreur"]["G"],
-                                                         f'PRONOTE server returned error code: {response.json()["Erreur"]["G"]} | {response.json()["Erreur"]["Titre"]}'))
-            log.info(f'Have you tried turning it off and on again? ERROR: {response.json()["Erreur"]["G"]} | {response.json()["Erreur"]["Titre"]}')
-            self.client._refresh()
+
+                raise PronoteAPIError(error_messages.get(r_json["Erreur"]["G"],
+                                                         f'Unknown error from pronote: {r_json["Erreur"]["G"]} | {r_json["Erreur"]["Titre"]}'))
+
+            log.info(f'Have you tried turning it off and on again? ERROR: {r_json["Erreur"]["G"]} | {r_json["Erreur"]["Titre"]}')
+            self.client.refresh()
             return self.client.communication.post(function_name, data, True)
 
         return response
@@ -425,7 +439,7 @@ class _Communication(object):
         if onload:
             onload_c = onload['onload'][14:-37]
         else:
-            if 'IP' in html:
+            if b'IP' in html:
                 raise PronoteAPIError('Your IP address is suspended.')
             raise PronoteAPIError("Page html is different than expected. Be sure that pronote_url is the direct url to your pronote page.")
         attributes = {}
@@ -439,14 +453,6 @@ class _Communication(object):
         return '/'.join(addr.split('/')[:-1]), '/'.join(addr.split('/')[-1:])
 
 
-def _create_random_string(length):
-    output = ''
-    for _ in range(length):
-        j = random.choice('ABCDEFGHIJKLMNOPQRSTUVabcdefghijklmnopqrstuv123456789')
-        output += j
-    return output
-
-
 def _enleverAlea(text):
     """Gets rid of the stupid thing that they did, idk what it really is for, but i guess it adds security"""
     sansalea = []
@@ -479,7 +485,7 @@ class _Encryption(object):
         """The encryption part of the API. You shouldn't have to use this normally."""
         # aes
         self.aes_iv = bytes(16)
-        self.aes_iv_temp = _create_random_string(16).encode()
+        self.aes_iv_temp = bytearray([random.randint(0, 255) for _ in range(16)])
         self.aes_key = MD5.new().digest()
         # rsa
         self.rsa_keys = {}
@@ -539,3 +545,8 @@ class PronoteAPIError(Exception):
 class CryptoError(PronoteAPIError):
     """Exception for known errors in the cryptography."""
     pass
+
+
+class ExpiredObject(PronoteAPIError):
+    """Raised when pronote returns error 22. (unknown object reference)"""
+    pass
